local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")
local faction = UnitFactionGroup("player")

function Capitalist:SanitizeOldData()
	-- FEATURE: DisallowEnchantingVellumRecipe
	-- With 4.0.3a, Enchanting Vellum became merchant-supplied for 8s.
	-- This is a significant discount from the cost used to make it with
	-- the Inscription recipe.  This is the only known example of such a
	-- case: merchant supplying a 'part'.  Until/unless more of these show
	-- up, it's simpler to just ignore the Inscription recipe rather than
	-- add the concept of "crafting vs. buying" throughout.
	local profession = L["Inscription"]
	local skillName = L["Enchanting Vellum"]
	if (self.db.profile.professions and
		self.db.profile.professions[profession] and
		self.db.profile.professions[profession][skillName]) then

		self.db.profile.professions[profession][skillName] = nil
		self:ClearValues()
	end
	if (self:GetProfile_Realm_Faction_Character_Profession_Skill(realm, faction, character, profession, skillName)) then
		self.db.profile[realm][faction][character][profession][skillName] = nil
	end
end

local hookPickupInventoryItem
local hookSpellTargetItem
local hookUseContainerItem
local hookUseItemByName

local LCI = LibStub("LibCraftInfo-1.0")

function Capitalist:GetExpansionId(spellId)
	local expansionId = LCI:GetCraftXPack(spellId)
	if (expansionId == nil) then
		-- assume Legion if unknown
		expansionId = 7
	end
	return expansionId
end
function Capitalist:GetProductLink(spellId)
	local productLink = select(2, GetItemInfo(LCI:GetCraftResultItem(spellId)))
	return productLink
end

function Capitalist:PrintCraftInfo(spellId)
	self:Print(L.expansions[self:GetExpansionId(spellId)] .. " - " .. self:GetProductLink(spellId))
end

function Capitalist:OnInitialize()
	--self:Print(L.expansions[self:GetExpansionId(22866)])
	--self:PrintCraftInfo(22866)
	--self:PrintCraftInfo(27033)
	--self:PrintCraftInfo(75597)
	--self:PrintCraftInfo(80719)
	--self:PrintCraftInfo(114757)
	--self:PrintCraftInfo(178550)
	--self:PrintCraftInfo(194735)
	--self:Print(L.expansions[self:GetExpansionId(194735)])
	--self:Print(self:GetExpansionId(194735))
	local defaults = {
			profile = {
				version = 0.11,
			
				margin = 100,
				minMargin = 100*10000,
				maxMargin = 300*10000,
				minProfit = 0*10000,
				showByDefault = false,
				cancelYellow = true,
				cancelBlue = true,
				cancelDuration = true,
				filters = {
					global = {
						Legion = true,
						WarlordsOfDraenor = false,
						MistsOfPandaria = false,
						Cataclysm = false,
						WrathOfTheLichKing = false,
						BurningCrusade = false,
						Classic = false,
					},
				},
			}
		}
	self.db = LibStub("AceDB-3.0"):New("CapitalistDB", defaults, true)
	
	self:ConvertUnfactionedServer(realm, faction, character)
	
	self:EnsureProfile_Realm_Faction_Character(realm, faction, character)

	if (not self.db.profile.optionsSpecific) then
		self.db.profile.optionsSpecific = {}
	end
	if (not self.db.profile.optionsSpecific[realm]) then
		self.db.profile.optionsSpecific[realm] = {}
	end
	if (not self.db.profile.optionsSpecific[realm][character]) then
		self.db.profile.optionsSpecific[realm][character] = {}
	end

	if (not self.db.profile.components) then
		self.db.profile.components = {}
	end
	if (not self.db.profile.professions) then
		self.db.profile.professions = {}
	end

	if (not self.db.profile.storage) then
		self.db.profile.storage = {}
	end
	if (not self.db.profile.storage[realm]) then
		self.db.profile.storage[realm] = {}
	end
	if (not self.db.profile.storage[realm][character]) then
		self.db.profile.storage[realm][character] = { AH = {}, bank = {}, inventory = {} }
	end

	if (not self.db.profile.debug) then
		self.db.profile.debug = {}
	end
	
	self:DebugMisc("Profile Version: "..self.db.profile.version)
	self:DebugMisc("Current Version: "..defaults.profile.version)
	
	self.db.profile.version = defaults.profile.version

	-- Late DebugEntry so that self.db is prepared
	self:DebugEntry("OnInitialize")

	self.updateHandler = CreateFrame("frame")
	self.updateHandler:Hide()
	self.elapsed = 0
	self.updateHandler:SetScript("OnUpdate", function(self, elapsed)
												self = Capitalist
--												self:DebugEntry("OnUpdate")
												self.elapsed = self.elapsed + elapsed
--												self:DebugMisc(self.mode)
												
												if (self.mode == "scanning") then
													self:OnUpdateScanning()
												elseif (self.mode == "posting") then
													self:OnUpdatePosting()
												elseif (self.mode == "componentsearching") then
													self:OnUpdateComponentSearching()
												else
													self:OnUpdateComponentScanning()
												end
--												self:DebugExit("OnUpdate")
											end)
	
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("AUCTION_HOUSE_CLOSED")
	self:RegisterEvent("AUCTION_HOUSE_SHOW")
	self:RegisterEvent("AUCTION_ITEM_LIST_UPDATE")
	self:RegisterEvent("AUCTION_OWNED_LIST_UPDATE")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("BANKFRAME_CLOSED")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("CHAT_MSG_SYSTEM")
	self:RegisterEvent("LOOT_OPENED")
	self:RegisterEvent("MERCHANT_CLOSED")
	self:RegisterEvent("MERCHANT_SHOW")
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("TRADE_SKILL_DETAILS_UPDATE")
	self:RegisterEvent("TRADE_SKILL_LIST_UPDATE")
	self:RegisterEvent("UI_ERROR_MESSAGE")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("UPDATE_TRADESKILL_RECAST")

	hooksecurefunc("PickupInventoryItem", hookPickupInventoryItem)
	hooksecurefunc("SpellTargetItem", hookSpellTargetItem)
	hooksecurefunc("UseContainerItem", hookUseContainerItem)
	hooksecurefunc("UseItemByName", hookUseItemByName)
		
	if (IsAddOnLoaded("Blizzard_AuctionUI")) then
		self:ADDON_LOADED("ADDON_LOADED", "Blizzard_AuctionUI")
	end
	if (IsAddOnLoaded("Blizzard_TradeSkillUI")) then
		self:ADDON_LOADED("ADDON_LOADED", "Blizzard_TradeSkillUI")
	end

	self:SanitizeOldData()

	self:ResetSkillCaches()

	self:EnforceProductsCannotHaveCostSetting()
	self:UpdateComponentValues()
	self:UpdateCombinedValues()

	self:BuildOptions()
	self:ApplyFilters()

	self:DebugExit("OnInitialize")
end

function Capitalist:ADDON_LOADED(event, addon)
	self:DebugEntry("ADDON_LOADED")

	if (addon == "Blizzard_AuctionUI") then
		self:CreateAuctionUI()
		self:CreateComponentUI()
		if (self.db.profile.showByDefault) then
			self:ToggleAuctionUI()
		end
	end
	if (addon == "Blizzard_TradeSkillUI") then
		self.restockButton = CreateFrame("button", "CapitalistRestockButton", TradeSkillFrame, "UIPanelButtonTemplate")
		self.restockButton:SetPoint("TOPRIGHT", TradeSkillFrame, "TOPRIGHT", -25, -2)
		self.restockButton:SetHeight(17)
		self.restockButton:SetWidth(60)
		self.restockButton:SetText("(C)")
		self.restockButton:SetScript("OnClick", function() Capitalist:Restock(self:GetCurrentTradeSkill()) end)
		self.restockButton:Show()

		self.showStackButton = CreateFrame("button", "CapitalistShowStackButton", TradeSkillFrame, "UIPanelButtonTemplate")
		self.showStackButton:SetPoint("TOPRIGHT", self.restockButton, "TOPLEFT", 0, 0)
		self.showStackButton:SetHeight(17)
		self.showStackButton:SetWidth(30)
		self.showStackButton:SetText("(R)")
		self.showStackButton:SetScript("OnClick", function() Capitalist:ShowRestockStack(self:GetCurrentTradeSkill()) end)
		self.showStackButton:Show()
		
		self.TradeSkillFrame_Show_Hook = TradeSkillFrame_Show
		TradeSkillFrame_Show =  function()
									self:DebugMisc(self:GetCurrentTradeSkill())
									Capitalist.TradeSkillFrame_Show_Hook()
									self:ScanTradeSkillFrame(self:GetCurrentTradeSkill())
								end
									
	end

	self:DebugExit("ADDON_LOADED")
end

function Capitalist:AUCTION_ITEM_LIST_UPDATE()
	self:DebugEntry("AUCTION_ITEM_LIST_UPDATE")

	if (self.mode == "scanning") then
		if (not self.ScanAnalyzed) then
			self:OnAuctionItemListUpdate()
		end
	end

	self:DebugExit("AUCTION_ITEM_LIST_UPDATE")
end

function Capitalist:AUCTION_HOUSE_CLOSED()
	self:DebugEntry("AUCTION_HOUSE_CLOSED")

	self:OnLeaveAuctionUI()

	self:HideAuctionPostingFrame()
	self:AuctionHouseProductListClear()

	self:DebugExit("AUCTION_HOUSE_CLOSED")
end

function Capitalist:HandleSlashCommand(...)
	self:DebugEntry("HandleSlashCommand")

	--local arg1, arg2, arg3, arg4, arg5, arg6 = ...
	local arg1 = ...
	self:DebugMisc(arg1)

	if ((arg1 == "?") or (arg1 == "h") or (arg1 == "help")) then
		self:ReportSyntax()
	elseif (string.sub(arg1, 1, 2) == "c ") then
		self:OutputItemCost(arg1.sub(arg1, 3))
	elseif (string.sub(arg1, 1, 5) == "cost ") then
		self:OutputItemCost(arg1.sub(arg1, 6))
	elseif (string.sub(arg1, 1, 2) == "f ") then
		self:FindProducts(arg1.sub(arg1, 3))
	elseif (string.sub(arg1, 1, 5) == "find ") then
		self:FindProducts(arg1.sub(arg1, 6))
	elseif ((arg1 == "m") or (arg1 == "missing")) then
		self:OutputMissingCosts()
	elseif ((arg1 == "p") or (arg1 == "professions")) then
		local oCharacter = self:GetProfile_Realm_Faction_Character(realm, faction, character)
		if (oCharacter) then
			for profession, professionData in pairs(oCharacter) do
				self:Print(profession)
			end
		end
	elseif ((arg1 == "r") or (arg1 == "restock")) then
		if (self:GetCurrentTradeSkill()) then
			self:Restock(self:GetCurrentTradeSkill())
		end	
	elseif (string.sub(arg1, 1, 2) == "s ") then
		self:OutputProductCraftingSequence(arg1.sub(arg1, 3))
	elseif (string.sub(arg1, 1, 9) == "sequence ") then
		self:OutputProductCraftingSequence(arg1.sub(arg1, 10))
	elseif ((arg1 == "t") or (arg1 == "top10")) then
		self:ReportTop10()
	elseif ((arg1 == "v") or (arg1 == "version")) then
		self:ReportVersion()
	else
		InterfaceOptionsFrame_OpenToCategory("Capitalist")
	end

	self:DebugExit("HandleSlashCommand")
end

function hookPickupInventoryItem(slot)
	Capitalist:DebugEntry("hookPickupInventoryItem")
	if (not UnitCastingInfo("player")) then
		Capitalist:DebugMisc(slot)
		if (slot) then
			Capitalist.spellTarget = GetInventoryItemLink("player", slot)
			Capitalist:DebugMisc(Capitalist.spellTarget)
		end
	end
	Capitalist:DebugExit("hookPickupInventoryItem")
end

function hookSpellTargetItem(itemString)
	Capitalist:DebugEntry("hookSpellTargetItemHook")
	if (not UnitCastingInfo("player")) then
		if (itemString) then
			local _, itemLink = GetItemInfo(itemString)
			if (itemLink) then
				Capitalist.spellTarget = itemLink
				Capitalist:DebugMisc(Capitalist.spellTarget)
			end
		end
	end
	Capitalist:DebugExit("hookSpellTargetItemHook")
end

function hookUseContainerItem(bag, slot)
	Capitalist:DebugEntry("hookUseContainerItem")
	if (not UnitCastingInfo("player")) then
		if (bag and slot) then
			Capitalist:DebugMisc(bag)
			Capitalist:DebugMisc(slot)
			Capitalist.spellTarget = GetContainerItemLink(bag, slot)
			Capitalist:DebugMisc(Capitalist.spellTarget)
		end
	end
	Capitalist:DebugExit("hookUseContainerItem")
end

function hookUseItemByName(itemString)
	Capitalist:DebugEntry("hookUseItemByName")
	if (not UnitCastingInfo("player")) then
		if (itemString) then
			local _, itemLink = GetItemInfo(itemString)
			if (itemLink) then
				Capitalist.spellTarget = itemLink
				Capitalist:DebugMisc(Capitalist.spellTarget)
			end
		end
	end
	Capitalist:DebugExit("hookUseItemByName")
end

function Capitalist:ConvertUnfactionedServer(realm, faction, character)
	Capitalist:DebugEntry("ConversionUnfactionedServer")

	self:EnsureProfile_Realm_Faction(realm, faction)
	if (self.db.profile and
		self.db.profile[realm] and
		self.db.profile[realm][character]) then 
		self:Print("Converting data from r99 to r100:")
	    self:Print("  found ("..realm..","..character..")")
		self:Print("  converting...")
	    local oCharacter = self.db.profile[realm][character]
        self.db.profile[realm .. "-" .. faction][character] = oCharacter
        self.db.profile[realm][character] = nil
        self:Print("  completed.")
    end
    
    Capitalist:DebugExit("ConversionUnfactionedServer")
end